@@ -5,29 +5,43 @@ import argparse
 import shutil
 import os
 from os.path import join
-import pkg_resources
-import invoice2data.pdftotext as pdftotext
-from invoice2data.template import read_templates
-from invoice2data.output import invoices_to_csv
 import logging
 
+from .input import pdftotext
+from .input import pdfminer
+from .input import tesseract
+
+from invoice2data.extract.loader import read_templates
+
+from .output import to_csv
+from .output import to_json
+from .output import to_xml
+
+
 logger = logging.getLogger(__name__)
 
 FILENAME = "{date} {desc}.pdf"
 
-def extract_data(invoicefile, templates=None, debug=False):
+input_mapping = {
+    'pdftotext': pdftotext,
+    'tesseract': tesseract,
+    'pdfminer': pdfminer,
+    }
+
+output_mapping = {
+    'csv': to_csv,
+    'json': to_json,
+    'xml': to_xml,
+
+    'none': None
+    }
+
+def extract_data(invoicefile, templates=None, input_module=pdftotext):
     if templates is None:
-        templates = read_templates(
-            pkg_resources.resource_filename('invoice2data', 'templates'))
-    
-    extracted_str = pdftotext.to_text(invoicefile).decode('utf-8')
+        templates = read_templates()
+
+    extracted_str = input_module.to_text(invoicefile).decode('utf-8')
 
-    charcount = len(extracted_str)
-    logger.debug('number of char in pdf2text extract: %d', charcount)
-    # Disable OCR for now.
-    #if charcount < 40:
-        #logger.info('Starting OCR')
-        #extracted_str = image_to_text.to_text(invoicefile)
     logger.debug('START pdftotext result ===========================')
     logger.debug(extracted_str)
     logger.debug('END pdftotext result =============================')
@@ -42,13 +56,23 @@ def extract_data(invoicefile, templates=None, debug=False):
     logger.error('No template for %s', invoicefile)
     return False
 
-def main():
-    "Take folder or single file and analyze each."
+def create_parser():
+    '''Returns argument parser '''
 
-    parser = argparse.ArgumentParser(description='Process some integers.')
+    parser = argparse.ArgumentParser(description='Extract structured data from PDF files and save to CSV or JSON.')
+
+    parser.add_argument('--input-reader', choices=input_mapping.keys(),
+                        default='pdftotext', help='Choose text extraction function. Default: pdftotext')
+
+
+    parser.add_argument('--output-format', choices=output_mapping.keys(),
+                        default='none', help='Choose output format. Default: none')
+
+    parser.add_argument('--output-name', '-o', dest='output_name', default='invoices-output',
+                        help='Custom name for output file. Extension is added based on chosen format.')
 
     parser.add_argument('--debug', dest='debug', action='store_true',
-                        help='Print debug information.')
+                        help='Enable debug information.')
 
     parser.add_argument('--copy', '-c', dest='copy',
                         help='Copy renamed PDFs to specified folder.')
@@ -58,17 +82,26 @@ def main():
     
     parser.add_argument('--exclude-built-in-templates', dest='exclude_built_in_templates',
                         default=False, help='Ignore built-in templates.', action="store_true")
-    
+
     parser.add_argument('input_files', type=argparse.FileType('r'), nargs='+',
                         help='File or directory to analyze.')
 
-    args = parser.parse_args()
+    return parser
+
+def main(args=None):
+    '''Take folder or single file and analyze each.'''
+    if args is None:
+        parser = create_parser()
+        args = parser.parse_args()
 
     if args.debug:
         logging.basicConfig(level=logging.DEBUG)
     else:
         logging.basicConfig(level=logging.INFO)
 
+    input_module = input_mapping[args.input_reader]
+    output_module = output_mapping[args.output_format]
+
     templates = []
     
     # Load templates from external folder if set.
@@ -77,11 +110,11 @@ def main():
 
     # Load internal templates, if not disabled.
     if not args.exclude_built_in_templates:
-        templates += read_templates(pkg_resources.resource_filename('invoice2data', 'templates'))
+        templates += read_templates()
     
     output = []
     for f in args.input_files:
-        res = extract_data(f.name, templates=templates)
+        res = extract_data(f.name, templates=templates, input_module=input_module)
         if res:
             logger.info(res)
             output.append(res)
@@ -90,7 +123,9 @@ def main():
                     date=res['date'].strftime('%Y-%m-%d'),
                     desc=res['desc'])
                 shutil.copyfile(f.name, join(args.copy, filename))
-    invoices_to_csv(output, 'invoices-output.csv')
+
+    if output_module is not None:
+        output_module.write_to_file(output, args.output_name)
 
 if __name__ == '__main__':
     main()
